import subprocess

from pyroute2 import IPDB


def get_default_iface_name():
    ip = IPDB()
    interface_name = ip.interfaces[ip.routes['default']['oif']].get('ifname')
    ip.release()
    return interface_name


def iptables_version():
    iptables_proc = subprocess.Popen(['iptables', '-L'], stderr=subprocess.PIPE, stdout=subprocess.DEVNULL)
    text = iptables_proc.stderr.read()
    if "Warning: iptables-legacy tables present, use iptables-legacy to see them" in str(text):
        return 'iptables-nft'
    else:
        return 'iptables'


def iptables_create_syntropy_chain(version='-nft'):
    try:
        # Check if already exists, if not - create
        subprocess.run([f'iptables{version}', '-N', 'SYNTROPY_CHAIN'], stderr=subprocess.DEVNULL)
        subprocess.run(
            [f'iptables{version}', '-C', 'FORWARD', '-s', '0.0.0.0/0', '-d', '0.0.0.0/0', '-j', 'SYNTROPY_CHAIN'],
            check=True,
            stderr=subprocess.DEVNULL
        )
    except subprocess.CalledProcessError:
        subprocess.run(
            [f'iptables{version}', '-I', 'FORWARD', '-s', '0.0.0.0/0', '-d', '0.0.0.0/0', '-j', 'SYNTROPY_CHAIN'],
            stderr=subprocess.DEVNULL,
            check=False
        )
    if version == '-nft':
        iptables_create_syntropy_chain(version='-legacy')


def add_iptable_rules(ips: list, version='-nft'):
    for ip in ips:
        try:
            # Check if already exists, if not - create
            subprocess.run(
                [f'iptables{version}', '-C', 'SYNTROPY_CHAIN', '-p', 'all', '-s', ip, '-j', 'ACCEPT'],
                check=True,
                stderr=subprocess.DEVNULL
            )
        except subprocess.CalledProcessError:
            subprocess.run(
                [f'iptables{version}', '-A', 'SYNTROPY_CHAIN', '-p', 'all', '-s', ip, '-j', 'ACCEPT'],
                check=False,
                stderr=subprocess.DEVNULL
            )
    if version == '-nft':
        add_iptable_rules(ips, version='-legacy')


def delete_iptable_rules(ips: list, version='-nft'):
    for ip in ips:
        subprocess.run(
            [f'iptables{version}', '-D', 'FORWARD', '-p', 'all', '-s', ip, '-j', 'ACCEPT'],
            check=False,
            stderr=subprocess.DEVNULL
        )
    if version == '-nft':
        delete_iptable_rules(ips, version='-legacy')


def add_iptables_forward(ifname, version='-nft'):
    try:
        # Check if already exists, if not - create
        subprocess.run(
            [f'iptables{version}', '-C', 'FORWARD', '-i', ifname, '-j', 'ACCEPT'],
            check=True,
            stderr=subprocess.DEVNULL
        )
    except subprocess.CalledProcessError:
        subprocess.run(
            [f'iptables{version}', '-A', 'FORWARD', '-i', ifname, '-j', 'ACCEPT'],
            check=False,
            stderr=subprocess.DEVNULL
        )
        subprocess.run(
            [f'iptables{version}', '-t', 'nat', '-A', 'POSTROUTING', '-o', get_default_iface_name(), '-j', 'MASQUERADE'],
            check=False,
            stderr=subprocess.DEVNULL
        )
    if version == '-nft':
        add_iptables_forward(ifname, version='-legacy')



#!/usr/bin/env python3
"""
Noia platform agent.
"""

__author__ = "SYNTROPY Network"
__version__ = "0.0.83"
__license__ = "MIT"

import os
import argparse
import atexit
import logging

from platform_agent.config.logger import configure_logger
from platform_agent.config.settings import Config, AGENT_PATH_TMP, ConfigException
from platform_agent.agent_websocket import WebSocketClient

from pyroute2 import WireGuard

logger = logging.getLogger()

def exit_handler():
    from pathlib import Path
    import shutil

    dirpath = Path(AGENT_PATH_TMP)
    if dirpath.exists() and dirpath.is_dir():
        shutil.rmtree(dirpath)


atexit.register(exit_handler)


def main(args=None):
    """ This is executed when run from the command line """

    try:
        WireGuard()
    except:
        # Wireguard module load failed
        pass

    parser = argparse.ArgumentParser()

    # Required positional argument
    parser.add_argument("run", help="Required parameter to start agent")

    # Optional verbosity counter (eg. -v, -vv, -vvv, etc.)
    parser.add_argument(
        "-v",
        "--verbose",
        action="count",
        default=0,
        help="Verbosity (-v, -vv, etc)")

    # Specify output of "--version"
    parser.add_argument(
        "--version",
        action="version",
        version="%(prog)s (version {version})".format(version=__version__))

    args = parser.parse_args()
    agent(args)


def agent(args):
    """ Main entry point of the app """
    if args.run:
        # Retrieving settings from config on start
        try:
            Config()
        except ConfigException as e:
            logger.error(f"[CONFIG]: {e}")
            return

        # Configuring logger globally
        configure_logger()

        # Initiating WS client
        client = WebSocketClient(
            os.environ.get('SYNTROPY_CONTROLLER_URL', 'controller-prod-platform-agents.syntropystack.com'),
            os.environ['SYNTROPY_AGENT_TOKEN']
        )

        # Starting WS client main thread
        client.start()


if __name__ == "__main__":
    """ This is executed when run from the command line """
    main()



import json
import logging
import threading
import time
import queue

from platform_agent.lib.ctime import now

logger = logging.getLogger()


class BatchSender(threading.Thread):

    def __init__(self, client):
        super().__init__()
        self.queue = queue.Queue()
        self.client = client
        self.stop_batch_send = threading.Event()
        self.wg = None
        self.daemon = True

        threading.Thread.__init__(self)

    def get_from_queue(self):
        payloads = {}
        t_end = time.time() + 10  # run for 10 second
        while time.time() < t_end:
            try:
                message = self.queue.get(timeout=0.1)
            except queue.Empty:
                continue
            msg_type = message['msg_type']
            if not payloads.get(msg_type):
                payloads[msg_type] = [message.get('data')]
            else:
                payloads[msg_type].append(message.get('data'))
        return payloads

    def run(self):
        while True:
            payloads = self.get_from_queue()
            for msg_type in payloads.keys():
                self.client.send(json.dumps({
                    'id': "ID." + str(now()),
                    'executed_at': now(),
                    "type": msg_type,
                    "data": payloads[msg_type]
                }))

    def join(self, timeout=None):
        self.stop_batch_send.set()
        super().join(timeout)

